@@ -85,9 +85,7 @@ utils::globalVariables(c("measurementEstimate","transitionEstimate", "C",
 #' \item \code{cumulative} - whether the produced forecast was cumulative or not.
 #' \item \code{actuals} - original data.
 #' \item \code{holdout} - holdout part of the original data.
-#' \item \code{iprob} - fitted and forecasted values of the probability of demand
-#' occurrence.
-#' \item \code{intermittent} - type of intermittent model fitted to the data.
+#' \item \code{imodel} - model of the class "iss" if intermittent model was estimated.
 #' \item \code{xreg} - provided vector or matrix of exogenous variables. If
 #' \code{xregDo="s"}, then this value will contain only selected exogenous variables.
 #' \item \code{updateX} - boolean, defining, if the states of exogenous variables
@@ -154,7 +152,7 @@ ges <- function(data, orders=c(1,1), lags=c(1,frequency(data)),
                 cfType=c("MSE","MAE","HAM","GMSTFE","MSTFE","MSEh","TFL"),
                 h=10, holdout=FALSE, cumulative=FALSE,
                 intervals=c("none","parametric","semiparametric","nonparametric"), level=0.95,
-                intermittent=c("none","auto","fixed","croston","tsb","sba"),
+                intermittent=c("none","auto","fixed","croston","tsb","sba"), imodel="MNN",
                 bounds=c("admissible","none"),
                 silent=c("none","all","graph","legend","output"),
                 xreg=NULL, xregDo=c("use","select"), initialX=NULL,
@@ -177,11 +175,8 @@ ges <- function(data, orders=c(1,1), lags=c(1,frequency(data)),
         else if(gregexpr("GES",model$model)==-1){
             stop("The provided model is not GES.",call.=FALSE);
         }
-        intermittent <- model$intermittent;
-        if(any(intermittent==c("p","provided"))){
-            warning("The provided model had predefined values of occurences for the holdout. We don't have them.",call.=FALSE);
-            warning("Switching to intermittent='auto'.",call.=FALSE);
-            intermittent <- "a";
+        if(!is.null(model$imodel)){
+            imodel <- model$imodel;
         }
         initial <- model$initial;
         persistence <- model$persistence;
@@ -203,7 +198,7 @@ ges <- function(data, orders=c(1,1), lags=c(1,frequency(data)),
 
 ##### Set environment for ssInput and make all the checks #####
     environment(ssInput) <- environment();
-    ssInput(modelType="ges",ParentEnvironment=environment());
+    ssInput("ges",ParentEnvironment=environment());
 
 ##### Initialise ges #####
 ElementsGES <- function(C){
@@ -408,7 +403,7 @@ CreatorGES <- function(silentText=FALSE,...){
     errors <- rep(NA,obsInsample);
 
 ##### Prepare exogenous variables #####
-    xregdata <- ssXreg(data=data, xreg=xreg, updateX=updateX,
+    xregdata <- ssXreg(data=data, xreg=xreg, updateX=updateX, ot=ot,
                        persistenceX=persistenceX, transitionX=transitionX, initialX=initialX,
                        obsInsample=obsInsample, obsAll=obsAll, obsStates=obsStates, maxlag=maxlag, h=h, silent=silentText);
 
@@ -453,11 +448,20 @@ CreatorGES <- function(silentText=FALSE,...){
 
 ##### Check number of observations vs number of max parameters #####
     if(obsNonzero <= nParamMax){
-        if(!silentText){
-            message(paste0("Number of non-zero observations is ",obsNonzero,
-                           ", while the number of parameters to estimate is ", nParamMax,"."));
+        if(xregDo=="select"){
+            if(obsNonzero <= (nParamMax - nParamExo)){
+                stop(paste0("Not enough observations for the reasonable fit. Number of parameters is ",
+                            nParamMax," while the number of observations is ",obsNonzero - nParamExo,"!"),call.=FALSE);
+            }
+            else{
+                warning(paste0("The potential number of exogenous variables is higher than the number of observations. ",
+                               "This may cause problems in the estimation."),call.=FALSE);
+            }
+        }
+        else{
+            stop(paste0("Not enough observations for the reasonable fit. Number of parameters is ",
+                        nParamMax," while the number of observations is ",obsNonzero,"!"),call.=FALSE);
         }
-        stop("Not enough observations. Can't fit the model you ask.",call.=FALSE);
     }
 
 ##### Preset values of matvt ######
@@ -537,7 +541,6 @@ CreatorGES <- function(silentText=FALSE,...){
         }
         if(iBest!=1){
             intermittent <- intermittentModelsPool[iBest];
-            intermittentModel <- intermittentModelsList[[iBest]];
             gesValues <- intermittentModelsList[[iBest]];
         }
         else{
@@ -675,6 +678,10 @@ CreatorGES <- function(silentText=FALSE,...){
         else{
             errormeasures <- errorMeasurer(y.holdout,y.for,y);
         }
+
+        if(cumulative){
+            y.holdout <- ts(sum(y.holdout),start=start(y.for),frequency=datafreq);
+        }
     }
     else{
         y.holdout <- NA;
@@ -725,7 +732,7 @@ CreatorGES <- function(silentText=FALSE,...){
         }
         else{
             graphmaker(actuals=data,forecast=y.for.new,fitted=y.fit,
-                       level=level,legend=!silentLegend,main=modelname,cumulative=cumulative);
+                       legend=!silentLegend,main=modelname,cumulative=cumulative);
         }
     }
 
@@ -736,7 +743,7 @@ CreatorGES <- function(silentText=FALSE,...){
                   nParam=nParam,
                   fitted=y.fit,forecast=y.for,lower=y.low,upper=y.high,residuals=errors,
                   errors=errors.mat,s2=s2,intervals=intervalsType,level=level,cumulative=cumulative,
-                  actuals=data,holdout=y.holdout,iprob=pt,intermittent=intermittent,
+                  actuals=data,holdout=y.holdout,imodel=imodel,
                   xreg=xreg,updateX=updateX,initialX=initialX,persistenceX=vecgX,transitionX=matFX,
                   ICs=ICs,logLik=logLik,cf=cfObjective,cfType=cfType,FI=FI,accuracy=errormeasures);
     return(structure(model,class="smooth"));
