@@ -114,9 +114,7 @@ utils::globalVariables(c("normalizer","constantValue","constantRequired","consta
 #' \item \code{cumulative} - whether the produced forecast was cumulative or not.
 #' \item \code{actuals} - the original data.
 #' \item \code{holdout} - the holdout part of the original data.
-#' \item \code{iprob} - the fitted and forecasted values of the probability of
-#' demand occurrence.
-#' \item \code{intermittent} - type of intermittent model fitted to the data.
+#' \item \code{imodel} - model of the class "iss" if intermittent model was estimated.
 #' \item \code{xreg} - provided vector or matrix of exogenous variables. If
 #' \code{xregDo="s"}, then this value will contain only selected exogenous
 #' variables.
@@ -195,7 +193,7 @@ ssarima <- function(data, orders=list(ar=c(0),i=c(1),ma=c(1)), lags=c(1),
                     cfType=c("MSE","MAE","HAM","GMSTFE","MSTFE","MSEh","TFL"),
                     h=10, holdout=FALSE, cumulative=FALSE,
                     intervals=c("none","parametric","semiparametric","nonparametric"), level=0.95,
-                    intermittent=c("none","auto","fixed","croston","tsb","sba"),
+                    intermittent=c("none","auto","fixed","croston","tsb","sba"), imodel="MNN",
                     bounds=c("admissible","none"),
                     silent=c("none","all","graph","legend","output"),
                     xreg=NULL, xregDo=c("use","select"), initialX=NULL,
@@ -222,11 +220,8 @@ ssarima <- function(data, orders=list(ar=c(0),i=c(1),ma=c(1)), lags=c(1),
 
 # If this is a normal ARIMA, do things
         if(any(unlist(gregexpr("combine",model$model))==-1)){
-            intermittent <- model$intermittent;
-            if(any(intermittent==c("p","provided"))){
-                warning("The provided model had predefined values of occurences for the holdout. We don't have them.",call.=FALSE);
-                warning("Switching to intermittent='auto'.",call.=FALSE);
-                intermittent <- "a";
+            if(!is.null(model$imodel)){
+                imodel <- model$imodel;
             }
             if(!is.null(model$initial)){
                 initial <- model$initial;
@@ -296,7 +291,7 @@ ssarima <- function(data, orders=list(ar=c(0),i=c(1),ma=c(1)), lags=c(1),
 
 ##### Set environment for ssInput and make all the checks #####
     environment(ssInput) <- environment();
-    ssInput(modelType="ssarima",ParentEnvironment=environment());
+    ssInput("ssarima",ParentEnvironment=environment());
 
 # Cost function for SSARIMA
 CF <- function(C){
@@ -502,7 +497,7 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
     errors <- rep(NA,obsInsample);
 
 ##### Prepare exogenous variables #####
-    xregdata <- ssXreg(data=data, xreg=xreg, updateX=updateX,
+    xregdata <- ssXreg(data=data, xreg=xreg, updateX=updateX, ot=ot,
                        persistenceX=persistenceX, transitionX=transitionX, initialX=initialX,
                        obsInsample=obsInsample, obsAll=obsAll, obsStates=obsStates, maxlag=maxlag, h=h, silent=silentText);
 
@@ -547,8 +542,20 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
 
 ##### Check number of observations vs number of max parameters #####
     if(obsNonzero <= nParamMax){
-        stop(paste0("Not enough observations for the reasonable fit. Number of parameters is ",
-                    nParamMax," while the number of observations is ",obsNonzero,"!"),call.=FALSE);
+        if(xregDo=="select"){
+            if(obsNonzero <= (nParamMax - nParamExo)){
+                stop(paste0("Not enough observations for the reasonable fit. Number of parameters is ",
+                            nParamMax," while the number of observations is ",obsNonzero - nParamExo,"!"),call.=FALSE);
+            }
+            else{
+                warning(paste0("The potential number of exogenous variables is higher than the number of observations. ",
+                               "This may cause problems in the estimation."),call.=FALSE);
+            }
+        }
+        else{
+            stop(paste0("Not enough observations for the reasonable fit. Number of parameters is ",
+                        nParamMax," while the number of observations is ",obsNonzero,"!"),call.=FALSE);
+        }
     }
 
 #####Start the calculations#####
@@ -608,7 +615,6 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         }
         if(iBest!=1){
             intermittent <- intermittentModelsPool[iBest];
-            intermittentModel <- intermittentModelsList[[iBest]];
             ssarimaValues <- intermittentModelsList[[iBest]];
         }
         else{
@@ -828,6 +834,10 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         else{
             errormeasures <- errorMeasurer(y.holdout,y.for,y);
         }
+
+        if(cumulative){
+            y.holdout <- ts(sum(y.holdout),start=start(y.for),frequency=datafreq);
+        }
     }
     else{
         y.holdout <- NA;
@@ -921,7 +931,7 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         }
         else{
             graphmaker(actuals=data,forecast=y.for.new,fitted=y.fit,
-                       level=level,legend=!silentLegend,main=modelname,cumulative=cumulative);
+                       legend=!silentLegend,main=modelname,cumulative=cumulative);
         }
     }
 
@@ -933,7 +943,7 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
                   nParam=nParam,
                   fitted=y.fit,forecast=y.for,lower=y.low,upper=y.high,residuals=errors,
                   errors=errors.mat,s2=s2,intervals=intervalsType,level=level,cumulative=cumulative,
-                  actuals=data,holdout=y.holdout,iprob=pt,intermittent=intermittent,
+                  actuals=data,holdout=y.holdout,imodel=imodel,
                   xreg=xreg,updateX=updateX,initialX=initialX,persistenceX=vecgX,transitionX=matFX,
                   ICs=ICs,logLik=logLik,cf=cfObjective,cfType=cfType,FI=FI,accuracy=errormeasures);
     return(structure(model,class="smooth"));
