@@ -6,7 +6,7 @@
 using namespace Rcpp;
 
 /* # Function is needed to estimate the correct error for ETS when multisteps model selection with r(matvt) is sorted out. */
-arma::mat matrixPower(arma::mat A, int power){
+arma::mat matrixPower(arma::mat const &A, int const &power){
     arma::mat B(A.n_rows, A.n_rows, arma::fill::eye);
 
     if(power!=0){
@@ -28,7 +28,7 @@ RcppExport SEXP matrixPowerWrap(SEXP matA, SEXP power){
 }
 
 /* # Function allows to multiply polinomails */
-arma::vec polyMult(arma::vec poly1, arma::vec poly2){
+arma::vec polyMult(arma::vec const &poly1, arma::vec const &poly2){
 
     int poly1Nonzero = arma::as_scalar(find(poly1,1,"last"));
     int poly2Nonzero = arma::as_scalar(find(poly2,1,"last"));
@@ -56,9 +56,38 @@ RcppExport SEXP polyMultwrap(SEXP polyVec1, SEXP polyVec2){
     return wrap(polyMult(poly1, poly2));
 }
 
+/* # Function returns value of CDF-based likelihood function for the whole series */
+double cdf(arma::vec const &vecYt, arma::vec const &vecYfit, arma::vec const &matErrors, char const &E){
+
+    double errorSD = arma::as_scalar(sqrt(mean(pow(matErrors,2))));
+    double CF = 0.0;
+    double CFbuffer;
+    int obs = vecYt.n_rows;
+    if(E=='A'){
+        for(int i=0; i<obs; ++i){
+            CFbuffer = log(R::pnorm(ceil(vecYt(i)), vecYfit(i), errorSD, 1, 0) - R::pnorm(ceil(vecYt(i))-1, vecYfit(i), errorSD, 1, 0));
+            // If CF is infinite, then this means that P(x<X)=1 for both cases.
+            // This is automatically transfered into log(1)
+            if(arma::is_finite(CFbuffer)){
+                CF += CFbuffer;
+            }
+        }
+    }
+    else{
+        for(int i=0; i<obs; ++i){
+            CFbuffer = log(R::plnorm(ceil(vecYt(i)), log(vecYfit(i)), errorSD, 1, 0) - R::plnorm(ceil(vecYt(i))-1, log(vecYfit(i)), errorSD, 1, 0));
+            if(arma::is_finite(CFbuffer)){
+                CF += CFbuffer;
+            }
+        }
+    }
+
+    return CF;
+}
+
 /* # Function returns multiplicative or additive error for scalar */
-double errorf(double yact, double yfit, char Etype){
-    if(Etype=='A'){
+double errorf(double const &yact, double &yfit, char const &E){
+    if(E=='A'){
         return yact - yfit;
     }
     else{
@@ -75,8 +104,8 @@ double errorf(double yact, double yfit, char Etype){
 }
 
 /* # Function is needed to estimate the correct error for ETS when multisteps model selection with r(matvt) is sorted out. */
-arma::mat errorvf(arma::mat yact, arma::mat yfit, char Etype){
-    if(Etype=='A'){
+arma::mat errorvf(arma::mat yact, arma::mat yfit, char const &E){
+    if(E=='A'){
         return yact - yfit;
     }
     else{
@@ -87,11 +116,11 @@ arma::mat errorvf(arma::mat yact, arma::mat yfit, char Etype){
 
 /* # Function returns value of w() -- y-fitted -- used in the measurement equation */
 // !!! Add matrixAt and xreg as arguments passed here and implement additive / multiplicative xreg!!!
-double wvalue(arma::vec vecVt, arma::rowvec rowvecW, char E, char T, char S,
-              arma::rowvec rowvecXt, arma::vec vecAt){
+double wvalue(arma::vec const &vecVt, arma::rowvec const &rowvecW, char const &E, char const &T, char const &S,
+              arma::rowvec const &rowvecXt, arma::vec const &vecAt){
 // vecVt is a vector here!
     double yfit = 0;
-    arma::mat matyfit;
+    arma::mat vecYfit;
 
     switch(S){
 // ZZN
@@ -99,10 +128,10 @@ double wvalue(arma::vec vecVt, arma::rowvec rowvecW, char E, char T, char S,
         switch(T){
         case 'N':
         case 'A':
-            matyfit = rowvecW * vecVt;
+            vecYfit = rowvecW * vecVt;
         break;
         case 'M':
-            matyfit = exp(rowvecW * log(vecVt));
+            vecYfit = exp(rowvecW * log(vecVt));
         break;
         }
     break;
@@ -111,10 +140,10 @@ double wvalue(arma::vec vecVt, arma::rowvec rowvecW, char E, char T, char S,
         switch(T){
         case 'N':
         case 'A':
-            matyfit = rowvecW * vecVt;
+            vecYfit = rowvecW * vecVt;
         break;
         case 'M':
-            matyfit = exp(rowvecW.cols(0,1) * log(vecVt.rows(0,1))) + vecVt(2);
+            vecYfit = exp(rowvecW.cols(0,1) * log(vecVt.rows(0,1))) + vecVt(2);
         break;
         }
     break;
@@ -123,10 +152,10 @@ double wvalue(arma::vec vecVt, arma::rowvec rowvecW, char E, char T, char S,
         switch(T){
         case 'N':
         case 'M':
-            matyfit = exp(rowvecW * log(vecVt));
+            vecYfit = exp(rowvecW * log(vecVt));
         break;
         case 'A':
-            matyfit = rowvecW.cols(0,1) * vecVt.rows(0,1) * vecVt(2);
+            vecYfit = rowvecW.cols(0,1) * vecVt.rows(0,1) * vecVt(2);
         break;
         }
     break;
@@ -134,10 +163,10 @@ double wvalue(arma::vec vecVt, arma::rowvec rowvecW, char E, char T, char S,
 
     switch(E){
         case 'A':
-            yfit = as_scalar(matyfit + rowvecXt * vecAt);
+            yfit = as_scalar(vecYfit + rowvecXt * vecAt);
         break;
         case 'M':
-            yfit = real(exp(log(std::complex<double>(as_scalar(matyfit))) + as_scalar(rowvecXt * vecAt)));
+            yfit = as_scalar(vecYfit * exp(rowvecXt * vecAt));
         break;
     }
 
@@ -146,8 +175,8 @@ double wvalue(arma::vec vecVt, arma::rowvec rowvecW, char E, char T, char S,
 
 /* # Function returns value of r() -- additive or multiplicative error -- used in the error term of measurement equation.
      This is mainly needed by sim.ets */
-double rvalue(arma::vec vecVt, arma::rowvec rowvecW, char E, char T, char S,
-              arma::rowvec rowvecXt, arma::vec vecAt){
+double rvalue(arma::vec const &vecVt, arma::rowvec const &rowvecW, char const &E, char const &T, char const &S,
+              arma::rowvec const &rowvecXt, arma::vec const &vecAt){
 
     switch(E){
 // MZZ
@@ -162,7 +191,7 @@ double rvalue(arma::vec vecVt, arma::rowvec rowvecW, char E, char T, char S,
 }
 
 /* # Function returns value of f() -- new states without the update -- used in the transition equation */
-arma::vec fvalue(arma::vec matrixVt, arma::mat matrixF, char T, char S){
+arma::vec fvalue(arma::vec const &matrixVt, arma::mat const &matrixF, char const T, char const S){
     arma::vec matrixVtnew = matrixVt;
 
     switch(S){
@@ -209,7 +238,7 @@ arma::vec fvalue(arma::vec matrixVt, arma::mat matrixF, char T, char S){
 }
 
 /* # Function returns value of g() -- the update of states -- used in components estimation for the persistence */
-arma::vec gvalue(arma::vec matrixVt, arma::mat matrixF, arma::mat rowvecW, char E, char T, char S){
+arma::vec gvalue(arma::vec const &matrixVt, arma::mat const &matrixF, arma::mat const &rowvecW, char const &E, char const &T, char const &S){
     arma::vec g(matrixVt.n_rows, arma::fill::ones);
 
 // AZZ
@@ -305,7 +334,7 @@ arma::vec gvalue(arma::vec matrixVt, arma::mat matrixF, arma::mat rowvecW, char
 
 /* # Function is needed for the proper xreg update in additive / multiplicative models.*/
 // Make sure that the provided vecXt is already in logs or whatever!
-arma::vec gXvalue(arma::vec vecXt, arma::vec vecGX, arma::vec error, char E){
+arma::vec gXvalue(arma::vec const &vecXt, arma::vec const &vecGX, arma::vec const &error, char const &E){
     arma::vec bufferforat(vecGX.n_rows);
 
     switch(E){
@@ -322,7 +351,7 @@ arma::vec gXvalue(arma::vec vecXt, arma::vec vecGX, arma::vec error, char E){
 }
 
 /* # Function is needed for the renormalisation of seasonal components. It should be done seasonal-wise.*/
-arma::mat normaliser(arma::mat Vt, int obsall, unsigned int maxlag, char S, char T){
+arma::mat normaliser(arma::mat Vt, int &obsall, unsigned int &maxlag, char const &S, char const &T){
 
     unsigned int nComponents = Vt.n_rows;
     double meanseason = 0;
@@ -657,12 +686,12 @@ RcppExport SEXP etsmatrices(SEXP matvt, SEXP vecg, SEXP phi, SEXP Cvalues, SEXP
 # and then in matF and other things.
 # Cvalues includes AR, MA, initials, constant, matrixAt, transitionX and persistenceX.
 */
-List polysos(arma::uvec arOrders, arma::uvec maOrders, arma::uvec iOrders, arma::uvec lags, unsigned int nComponents,
-             arma::vec arValues, arma::vec maValues, double constValue, arma::vec C,
-             arma::mat matrixVt, arma::vec vecG, arma::mat matrixF,
-             char fitterType, int nexo, arma::mat matrixAt, arma::mat matrixFX, arma::vec vecGX,
-             bool arEstimate, bool maEstimate, bool constRequired, bool constEstimate,
-             bool xregEstimate, bool wild, bool fXEstimate, bool gXEstimate, bool initialXEstimate){
+List polysos(arma::uvec const &arOrders, arma::uvec const &maOrders, arma::uvec const &iOrders, arma::uvec const &lags, unsigned int const &nComponents,
+             arma::vec const &arValues, arma::vec const &maValues, double const &constValue, arma::vec const &C,
+             arma::mat &matrixVt, arma::vec &vecG, arma::mat &matrixF,
+             char const &fitterType, int const &nexo, arma::mat &matrixAt, arma::mat &matrixFX, arma::vec &vecGX,
+             bool const &arEstimate, bool const &maEstimate, bool const &constRequired, bool const &constEstimate,
+             bool const &xregEstimate, bool const &wild, bool const &fXEstimate, bool const &gXEstimate, bool const &initialXEstimate){
 
 // Form matrices with parameters, that are then used for polynomial multiplication
     arma::mat arParameters(max(arOrders % lags)+1, arOrders.n_elem, arma::fill::zeros);
@@ -894,10 +923,10 @@ RcppExport SEXP polysoswrap(SEXP ARorders, SEXP MAorders, SEXP Iorders, SEXP ARI
                         xregEstimate, wild, fXEstimate, gXEstimate, initialXEstimate));
 }
 
-
-List fitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arma::vec vecYt, arma::vec vecG,
-             arma::uvec lags, char E, char T, char S,
-             arma::mat matrixXt, arma::mat matrixAt, arma::mat matrixFX, arma::vec vecGX, arma::vec vecOt) {
+// # Fitter for univariate models
+List fitter(arma::mat &matrixVt, arma::mat const &matrixF, arma::rowvec const &rowvecW, arma::vec const &vecYt, arma::vec const &vecG,
+            arma::uvec &lags, char const &E, char const &T, char const &S,
+            arma::mat const &matrixXt, arma::mat &matrixAt, arma::mat const &matrixFX, arma::vec const &vecGX, arma::vec const &vecOt){
     /* # matrixVt should have a length of obs + maxlag.
     * # rowvecW should have 1 row.
     * # matgt should be a vector
@@ -924,21 +953,21 @@ List fitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arma::v
 
     arma::uvec lagrows(lagslength, arma::fill::zeros);
 
-    arma::vec matyfit(obs, arma::fill::zeros);
-    arma::vec materrors(obs, arma::fill::zeros);
+    arma::vec vecYfit(obs, arma::fill::zeros);
+    arma::vec matErrors(obs, arma::fill::zeros);
     arma::vec bufferforat(vecGX.n_rows);
 
     for (unsigned int i=maxlag; i<obs+maxlag; i=i+1) {
         lagrows = i * nComponents - lags + nComponents - 1;
 
 /* # Measurement equation and the error term */
-        matyfit.row(i-maxlag) = vecOt(i-maxlag) * wvalue(matrixVt(lagrows), rowvecW, E, T, S,
+        vecYfit.row(i-maxlag) = vecOt(i-maxlag) * wvalue(matrixVt(lagrows), rowvecW, E, T, S,
                                                          matrixXt.row(i-maxlag), matrixAt.col(i-1));
-        materrors(i-maxlag) = errorf(vecYt(i-maxlag), matyfit(i-maxlag), E);
+        matErrors(i-maxlag) = errorf(vecYt(i-maxlag), vecYfit(i-maxlag), E);
 
 /* # Transition equation */
         matrixVt.col(i) = fvalue(matrixVt(lagrows), matrixF, T, S) +
-                          gvalue(matrixVt(lagrows), matrixF, rowvecW, E, T, S) % vecG * materrors(i-maxlag);
+                          gvalue(matrixVt(lagrows), matrixF, rowvecW, E, T, S) % vecG * matErrors(i-maxlag);
 
 /* Failsafe for cases when unreasonable value for state vector was produced */
         if(!matrixVt.col(i).is_finite()){
@@ -962,7 +991,7 @@ List fitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arma::v
         }
 
 /* # Transition equation for xreg */
-        bufferforat = gXvalue(matrixXtTrans.col(i-maxlag), vecGX, materrors.row(i-maxlag), E);
+        bufferforat = gXvalue(matrixXtTrans.col(i-maxlag), vecGX, matErrors.row(i-maxlag), E);
         matrixAt.col(i) = matrixFX * matrixAt.col(i-1) + bufferforat;
     }
 
@@ -972,16 +1001,17 @@ List fitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arma::v
         matrixAt.col(i) = matrixFX * matrixAt.col(i-1);
     }
 
-    matrixVt = matrixVt.t();
-    matrixAt = matrixAt.t();
+    // matrixVt = matrixVt.t();
+    // matrixAt = matrixAt.t();
 
-    return List::create(Named("matvt") = matrixVt, Named("yfit") = matyfit,
-                        Named("errors") = materrors, Named("matat") = matrixAt);
+    return List::create(Named("matvt") = matrixVt.t(), Named("yfit") = vecYfit,
+                        Named("errors") = matErrors, Named("matat") = matrixAt.t());
 }
 
-List backfitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arma::vec vecYt, arma::vec vecG,
-                arma::uvec lags, char E, char T, char S,
-                arma::mat matrixXt, arma::mat matrixAt, arma::mat matrixFX, arma::vec vecGX, arma::vec vecOt) {
+// # Backfitter for univariate models
+List backfitter(arma::mat &matrixVt, arma::mat const &matrixF, arma::rowvec const &rowvecW, arma::vec const &vecYt, arma::vec const &vecG,
+                arma::uvec &lags, char const &E, char const &T, char const &S,
+                arma::mat const &matrixXt, arma::mat &matrixAt, arma::mat const &matrixFX, arma::vec const &vecGX, arma::vec const &vecOt){
     /* # matrixVt should have a length of obs + maxlag.
     * # rowvecW should have 1 row.
     * # matgt should be a vector
@@ -996,6 +1026,17 @@ List backfitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arm
     matrixAt = matrixAt.t();
     arma::mat matrixXtTrans = matrixXt.t();
 
+    // Inverse transition matrix for backcasting
+    // arma::mat matrixFInv;
+    // if(!arma::inv(matrixFInv,matrixF)){
+    //     matrixFInv = matrixF.t();
+    //     matrixFInv(0,0) = 0;
+    //     matrixFInv.col(matrixFInv.n_cols-1) = flipud(matrixF.col(0));
+    //     // matrixFInv.col(matrixFInv.n_cols-1) = matrixF.col(0);
+    // }
+    // // arma::vec vecGInv = vecG - matrixF.col(0) + matrixFInv.col(0);
+    // arma::rowvec rowvecWInv = rowvecW * matrixFInv * matrixFInv;
+
     int obs = vecYt.n_rows;
     int obsall = matrixVt.n_cols;
     unsigned int nComponents = matrixVt.n_rows;
@@ -1011,8 +1052,8 @@ List backfitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arm
 
     arma::uvec lagrows(lagslength, arma::fill::zeros);
 
-    arma::vec matyfit(obs, arma::fill::zeros);
-    arma::vec materrors(obs, arma::fill::zeros);
+    arma::vec vecYfit(obs, arma::fill::zeros);
+    arma::vec matErrors(obs, arma::fill::zeros);
     arma::vec bufferforat(vecGX.n_rows);
 
     for(int j=0; j<=nloops; j=j+1){
@@ -1021,13 +1062,13 @@ List backfitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arm
             lagrows = i * nComponents - (lags + lagsModifier) + nComponents - 1;
 
 /* # Measurement equation and the error term */
-            matyfit.row(i-maxlag) = vecOt(i-maxlag) * wvalue(matrixVt(lagrows), rowvecW, E, T, S,
+            vecYfit.row(i-maxlag) = vecOt(i-maxlag) * wvalue(matrixVt(lagrows), rowvecW, E, T, S,
                                                              matrixXt.row(i-maxlag), matrixAt.col(i-1));
-            materrors(i-maxlag) = errorf(vecYt(i-maxlag), matyfit(i-maxlag), E);
+            matErrors(i-maxlag) = errorf(vecYt(i-maxlag), vecYfit(i-maxlag), E);
 
 /* # Transition equation */
             matrixVt.col(i) = fvalue(matrixVt(lagrows), matrixF, T, S) +
-                              gvalue(matrixVt(lagrows), matrixF, rowvecW, E, T, S) % vecG * materrors(i-maxlag);
+                              gvalue(matrixVt(lagrows), matrixF, rowvecW, E, T, S) % vecG * matErrors(i-maxlag);
 
 /* Failsafe for cases when unreasonable value for state vector was produced */
             if(!matrixVt.col(i).is_finite()){
@@ -1051,7 +1092,7 @@ List backfitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arm
             }
 
 /* # Transition equation for xreg */
-            bufferforat = gXvalue(matrixXtTrans.col(i-maxlag), vecGX, materrors.row(i-maxlag), E);
+            bufferforat = gXvalue(matrixXtTrans.col(i-maxlag), vecGX, matErrors.row(i-maxlag), E);
             matrixAt.col(i) = matrixFX * matrixAt.col(i-1) + bufferforat;
         }
 
@@ -1071,13 +1112,13 @@ List backfitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arm
             lagrows = i * nComponents + lags - lagsModifier + nComponents - 1;
 
 /* # Measurement equation and the error term */
-            matyfit.row(i-maxlag) = vecOt(i-maxlag) * wvalue(matrixVt(lagrows), rowvecW, E, T, S,
+            vecYfit.row(i-maxlag) = vecOt(i-maxlag) * wvalue(matrixVt(lagrows), rowvecW, E, T, S,
                                                              matrixXt.row(i-maxlag), matrixAt.col(i+1));
-            materrors(i-maxlag) = errorf(vecYt(i-maxlag), matyfit(i-maxlag), E);
+            matErrors(i-maxlag) = errorf(vecYt(i-maxlag), vecYfit(i-maxlag), E);
 
 /* # Transition equation */
             matrixVt.col(i) = fvalue(matrixVt(lagrows), matrixF, T, S) +
-                              gvalue(matrixVt(lagrows), matrixF, rowvecW, E, T, S) % vecG * materrors(i-maxlag);
+                              gvalue(matrixVt(lagrows), matrixF, rowvecW, E, T, S) % vecG * matErrors(i-maxlag);
 
 /* Failsafe for cases when unreasonable value for state vector was produced */
             if(!matrixVt.col(i).is_finite()){
@@ -1096,7 +1137,7 @@ List backfitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arm
 /* Skipping renormalisation of components in backcasting */
 
 /* # Transition equation for xreg */
-            bufferforat = gXvalue(matrixXtTrans.col(i-maxlag), vecGX, materrors.row(i-maxlag), E);
+            bufferforat = gXvalue(matrixXtTrans.col(i-maxlag), vecGX, matErrors.row(i-maxlag), E);
             matrixAt.col(i) = matrixFX * matrixAt.col(i+1) + bufferforat;
         }
 /* # Fill in the head of the matrices */
@@ -1107,11 +1148,11 @@ List backfitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arm
         }
     }
 
-    matrixVt = matrixVt.t();
-    matrixAt = matrixAt.t();
+    // matrixVt = matrixVt.t();
+    // matrixAt = matrixAt.t();
 
-    return List::create(Named("matvt") = matrixVt, Named("yfit") = matyfit,
-                        Named("errors") = materrors, Named("matat") = matrixAt);
+    return List::create(Named("matvt") = matrixVt.t(), Named("yfit") = vecYfit,
+                        Named("errors") = matErrors, Named("matat") = matrixAt.t());
 }
 
 /* # Wrapper for fitter */
@@ -1119,6 +1160,7 @@ List backfitter(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arm
 RcppExport SEXP fitterwrap(SEXP matvt, SEXP matF, SEXP matw, SEXP yt, SEXP vecg,
                             SEXP modellags, SEXP Etype, SEXP Ttype, SEXP Stype, SEXP fittertype,
                             SEXP matxt, SEXP matat, SEXP matFX, SEXP vecgX, SEXP ot) {
+
     NumericMatrix matvt_n(matvt);
     arma::mat matrixVt(matvt_n.begin(), matvt_n.nrow(), matvt_n.ncol());
 
@@ -1171,9 +1213,9 @@ RcppExport SEXP fitterwrap(SEXP matvt, SEXP matF, SEXP matw, SEXP yt, SEXP vecg,
 }
 
 /* # Function produces the point forecasts for the specified model */
-arma::mat forecaster(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW,
-                     unsigned int hor, char E, char T, char S, arma::uvec lags,
-                     arma::mat matrixXt, arma::mat matrixAt, arma::mat matrixFX) {
+arma::mat forecaster(arma::mat matrixVt, arma::mat const &matrixF, arma::rowvec const &rowvecW,
+                     unsigned int const &hor, char const &E, char const &T, char const &S, arma::uvec lags,
+                     arma::mat matrixXt, arma::mat matrixAt, arma::mat const &matrixFX){
     int lagslength = lags.n_rows;
     unsigned int maxlag = max(lags);
     unsigned int hh = hor + maxlag;
@@ -1238,27 +1280,27 @@ RcppExport SEXP forecasterwrap(SEXP matvt, SEXP matF, SEXP matw,
 }
 
 /* # Function produces matrix of errors based on multisteps forecast */
-arma::mat errorer(arma::mat matrixVt, arma::mat matrixF, arma::mat rowvecW, arma::mat vecYt,
-                  int hor, char E, char T, char S, arma::uvec lags,
-                  arma::mat matrixXt, arma::mat matrixAt, arma::mat matrixFX, arma::vec vecOt){
+arma::mat errorer(arma::mat const &matrixVt, arma::mat const &matrixF, arma::mat const &rowvecW, arma::mat const &vecYt,
+                  int const &hor, char const &E, char const &T, char const &S, arma::uvec const &lags,
+                  arma::mat const &matrixXt, arma::mat const &matrixAt, arma::mat const &matrixFX, arma::vec const &vecOt){
     int obs = vecYt.n_rows;
     int hh = 0;
-    arma::mat materrors(obs+hor-1, hor, arma::fill::zeros);
+    arma::mat matErrors(obs+hor-1, hor, arma::fill::zeros);
     unsigned int maxlag = max(lags);
 
     for(int i = 0; i < obs; i=i+1){
         hh = std::min(hor, obs-i);
-        materrors.submat(hor-1+i, 0, hor-1+i, hh-1) = trans(vecOt.rows(i, i+hh-1) % errorvf(vecYt.rows(i, i+hh-1),
+        matErrors.submat(hor-1+i, 0, hor-1+i, hh-1) = arma::trans(vecOt.rows(i, i+hh-1) % errorvf(vecYt.rows(i, i+hh-1),
             forecaster(matrixVt.rows(i,i+maxlag-1), matrixF, rowvecW, hh, E, T, S, lags, matrixXt.rows(i, i+hh-1),
                 matrixAt.rows(i, i+hh-1), matrixFX), E));
     }
 
 // Fix for GV in order to perform better in the sides of the series
     for(int i=0; i<(hor-1); i=i+1){
-        materrors.submat((hor-2)-(i),i+1,(hor-2)-(i),hor-1) = materrors.submat(hor-1,0,hor-1,hor-i-2) * sqrt(1.0+i);
+        matErrors.submat((hor-2)-(i),i+1,(hor-2)-(i),hor-1) = matErrors.submat(hor-1,0,hor-1,hor-i-2) * sqrt(1.0+i);
     }
 
-    return materrors;
+    return matErrors;
 }
 
 /* # Wrapper for errorer */
@@ -1315,14 +1357,15 @@ int CFtypeswitch (std::string const& CFtype) {
     if (CFtype == "aGMSTFE") return 9;
     if (CFtype == "aMSTFE") return 10;
     if (CFtype == "aMSEh") return 11;
+    if (CFtype == "Rounded") return 12;
     else return 7;
 }
 
 /* # Function returns the chosen Cost Function based on the chosen model and produced errors */
-double optimizer(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, arma::vec vecYt, arma::vec vecG,
-                 unsigned int hor, arma::uvec lags, char E, char T, char S,
-                 bool multi, std::string CFtype, double normalize, char fitterType,
-                 arma::mat  matrixXt, arma::mat matrixAt, arma::mat matrixFX, arma::vec vecGX, arma::vec vecOt){
+double optimizer(arma::mat &matrixVt, arma::mat const &matrixF, arma::rowvec const &rowvecW, arma::vec const &vecYt, arma::vec const &vecG,
+                 unsigned int const &hor, arma::uvec &lags, char const &E, char const &T, char const &S,
+                 bool const &multi, std::string const &CFtype, double const &normalize, char const &fitterType,
+                 arma::mat const &matrixXt, arma::mat &matrixAt, arma::mat const &matrixFX, arma::vec const &vecGX, arma::vec const &vecOt){
 // # Make decomposition functions shut up!
     std::ostream nullstream(0);
     arma::set_stream_err2(nullstream);
@@ -1354,34 +1397,34 @@ double optimizer(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, ar
     NumericMatrix matrixAtfromfit = as<NumericMatrix>(fitting["matat"]);
     matrixAt = as<arma::mat>(matrixAtfromfit);
 
-    arma::mat materrors;
+    arma::mat matErrors;
 
     arma::vec veccij(hor, arma::fill::ones);
     arma::mat matrixSigma(hor, hor, arma::fill::eye);
 
     if((multi==true) & (CFtypeswitch(CFtype)<=7)){
-        materrors = errorer(matrixVt, matrixF, rowvecW, vecYt, hor, E, T, S, lags, matrixXt, matrixAt, matrixFX, vecOt);
+        matErrors = errorer(matrixVt, matrixF, rowvecW, vecYt, hor, E, T, S, lags, matrixXt, matrixAt, matrixFX, vecOt);
         if(E=='M'){
-            materrors = log(1 + materrors);
-            materrors.elem(arma::find_nonfinite(materrors)).fill(1e10);
+            matErrors = log(1 + matErrors);
+            matErrors.elem(arma::find_nonfinite(matErrors)).fill(1e10);
 
 // This correction is needed in order to take the correct number of observations in the error matrix
             yactsum = yactsum / obs * matobs;
         }
     }
     else{
-        arma::mat materrorsfromfit(errorsfromfit.begin(), errorsfromfit.nrow(), errorsfromfit.ncol(), false);
-        materrors = materrorsfromfit;
-        materrors = materrors.elem(nonzeroes);
+        arma::mat matErrorsfromfit(errorsfromfit.begin(), errorsfromfit.nrow(), errorsfromfit.ncol(), false);
+        matErrors = matErrorsfromfit;
+        matErrors = matErrors.elem(nonzeroes);
         if(E=='M'){
-            materrors = log(1 + materrors);
+            matErrors = log(1 + matErrors);
         }
     }
 
-    if(CFtypeswitch(CFtype)>7){
+    if((CFtypeswitch(CFtype)>7) & (CFtypeswitch(CFtype)<12)){
 // Form vector for basic values and matrix Mu
         for(unsigned int i=1; i<hor; ++i){
-            veccij(i) = as_scalar(rowvecW * matrixPower(matrixF,i) * vecG);
+            veccij(i) = arma::as_scalar(rowvecW * matrixPower(matrixF,i) * vecG);
         }
 
 // Fill in the diagonal of Sigma matrix
@@ -1407,111 +1450,122 @@ double optimizer(arma::mat matrixVt, arma::mat matrixF, arma::rowvec rowvecW, ar
         }
     }
 
+    // Thes lines are needed for Rounded CF
+    arma::vec vecYfit;
+    if(CFtypeswitch(CFtype)==12){
+        NumericMatrix yfitfromfit = as<NumericMatrix>(fitting["yfit"]);
+        vecYfit = as<arma::vec>(yfitfromfit);
+    }
+
     switch(E){
     case 'M':
         switch(CFtypeswitch(CFtype)){
         case 1:
             try{
-                CFres = double(log(arma::prod(eig_sym(trans(materrors) * (materrors) / matobs))));
+                CFres = double(log(arma::prod(eig_sym(trans(matErrors) * (matErrors) / matobs))));
             }
             catch(const std::runtime_error){
-                CFres = double(log(arma::det(arma::trans(materrors) * materrors / double(matobs))));
+                CFres = double(log(arma::det(arma::trans(matErrors) * matErrors / double(matobs))));
             }
             CFres = CFres + (2 / double(matobs)) * double(hor) * yactsum;
         break;
         case 2:
-            CFres = arma::as_scalar(sum(log(sum(pow(materrors,2)) / double(matobs)), 1))
+            CFres = arma::as_scalar(sum(log(sum(pow(matErrors,2)) / double(matobs)), 1))
                     + (2 / double(obs)) * double(hor) * yactsum;
         break;
 // no exp is the temporary fix for very strange behaviour of MAM type models
         case 3:
-            CFres = arma::as_scalar(sum(sum(pow(materrors,2)) / double(matobs), 1)
+            CFres = arma::as_scalar(sum(sum(pow(matErrors,2)) / double(matobs), 1)
                         + (2 / double(obs)) * double(hor) * yactsum);
         break;
         case 4:
-            CFres = arma::as_scalar(exp(log(sum(pow(materrors.col(hor-1),2)) / double(matobs))
+            CFres = arma::as_scalar(exp(log(sum(pow(matErrors.col(hor-1),2)) / double(matobs))
                                         + (2 / double(obs)) * yactsum));
         break;
         case 5:
-            CFres = arma::as_scalar(exp(log(mean(abs(materrors))) + (2 / double(obs)) * yactsum));
+            CFres = arma::as_scalar(exp(log(mean(abs(matErrors))) + (2 / double(obs)) * yactsum));
         break;
         case 6:
-            CFres = arma::as_scalar(exp(log(mean(sqrt(abs(materrors)))) + (2 / double(obs)) * yactsum));
+            CFres = arma::as_scalar(exp(log(mean(sqrt(abs(matErrors)))) + (2 / double(obs)) * yactsum));
         break;
         case 7:
-            CFres = arma::as_scalar(exp(log(mean(pow(materrors,2))) + (2 / double(obs)) * yactsum));
+            CFres = arma::as_scalar(exp(log(mean(pow(matErrors,2))) + (2 / double(obs)) * yactsum));
         break;
         case 8:
         case 9:
             try{
-                CFres = double(log(arma::prod(eig_sym(as_scalar(mean(pow(materrors / normalize,2))) * matrixSigma
+                CFres = double(log(arma::prod(eig_sym(as_scalar(mean(pow(matErrors / normalize,2))) * matrixSigma
                                                           ))) + hor*log(pow(normalize,2)));
             }
             catch(const std::runtime_error){
-                CFres = log(arma::det(as_scalar(mean(pow(materrors / normalize,2))) * matrixSigma
+                CFres = log(arma::det(as_scalar(mean(pow(matErrors / normalize,2))) * matrixSigma
                                           )) + hor*log(pow(normalize,2));
             }
             CFres = CFres + (2 / double(matobs)) * double(hor) * yactsum;
         break;
         case 10:
-            CFres = arma::trace(as_scalar(mean(pow(materrors,2))) * matrixSigma
+            CFres = arma::trace(as_scalar(mean(pow(matErrors,2))) * matrixSigma
                                     );
             CFres = CFres + (2 / double(matobs)) * double(hor) * yactsum;
         break;
         case 11:
-            CFres = (as_scalar(mean(pow(materrors,2))) * matrixSigma(hor-1,hor-1)
-                                        );
+            CFres = (as_scalar(mean(pow(matErrors,2))) * matrixSigma(hor-1,hor-1));
             CFres = CFres + (2 / double(matobs)) * double(hor) * yactsum;
+        break;
+        case 12:
+            CFres = -cdf(vecYt.elem(nonzeroes), vecYfit.elem(nonzeroes), matErrors, E);
         }
     break;
     case 'A':
         switch(CFtypeswitch(CFtype)){
         case 1:
             try{
-                CFres = double(log(arma::prod(eig_sym(trans(materrors / normalize) * (materrors / normalize) / matobs))) +
+                CFres = double(log(arma::prod(eig_sym(trans(matErrors / normalize) * (matErrors / normalize) / matobs))) +
                     hor * log(pow(normalize,2)));
             }
             catch(const std::runtime_error){
-                CFres = double(log(arma::det(arma::trans(materrors / normalize) * (materrors / normalize) / matobs)) +
+                CFres = double(log(arma::det(arma::trans(matErrors / normalize) * (matErrors / normalize) / matobs)) +
                     hor * log(pow(normalize,2)));
             }
         break;
         case 2:
-            CFres = arma::as_scalar(sum(log(sum(pow(materrors,2)) / double(matobs)), 1));
+            CFres = arma::as_scalar(sum(log(sum(pow(matErrors,2)) / double(matobs)), 1));
         break;
         case 3:
-            CFres = arma::as_scalar(sum(sum(pow(materrors,2)) / double(matobs), 1));
+            CFres = arma::as_scalar(sum(sum(pow(matErrors,2)) / double(matobs), 1));
         break;
         case 4:
-            CFres = arma::as_scalar(sum(pow(materrors.col(hor-1),2)) / double(matobs));
+            CFres = arma::as_scalar(sum(pow(matErrors.col(hor-1),2)) / double(matobs));
         break;
         case 5:
-            CFres = arma::as_scalar(mean(abs(materrors)));
+            CFres = arma::as_scalar(mean(abs(matErrors)));
         break;
         case 6:
-            CFres = arma::as_scalar(mean(sqrt(abs(materrors))));
+            CFres = arma::as_scalar(mean(sqrt(abs(matErrors))));
         break;
         case 7:
-            CFres = arma::as_scalar(mean(pow(materrors,2)));
+            CFres = arma::as_scalar(mean(pow(matErrors,2)));
         break;
         case 8:
         case 9:
             try{
-                CFres = double(log(arma::prod(eig_sym(as_scalar(mean(pow(materrors / normalize,2))) * matrixSigma
+                CFres = double(log(arma::prod(eig_sym(as_scalar(mean(pow(matErrors / normalize,2))) * matrixSigma
                                                           ))) + hor*log(pow(normalize,2)));
             }
             catch(const std::runtime_error){
-                CFres = log(arma::det(as_scalar(mean(pow(materrors / normalize,2))) * matrixSigma
+                CFres = log(arma::det(as_scalar(mean(pow(matErrors / normalize,2))) * matrixSigma
                                           )) + hor*log(pow(normalize,2));
             }
         break;
         case 10:
-            CFres = arma::trace(as_scalar(mean(pow(materrors,2))) * matrixSigma
+            CFres = arma::trace(as_scalar(mean(pow(matErrors,2))) * matrixSigma
                                     );
         break;
         case 11:
-            CFres = (as_scalar(mean(pow(materrors,2))) * matrixSigma(hor-1,hor-1)
-                                        );
+            CFres = (as_scalar(mean(pow(matErrors,2))) * matrixSigma(hor-1,hor-1));
+        break;
+        case 12:
+            CFres = -cdf(vecYt.elem(nonzeroes), vecYfit.elem(nonzeroes), matErrors, E);
         }
     }
     return CFres;
@@ -1636,7 +1690,6 @@ RcppExport SEXP costfunc(SEXP matvt, SEXP matF, SEXP matw, SEXP yt, SEXP vecg,
 
 // Values needed for eigenvalues calculation
     arma::cx_vec eigval;
-    arma::mat matrixD = matrixF;
 
     if(boundtype=='u'){
 // alpha in (0,1)
@@ -1863,10 +1916,10 @@ RcppExport SEXP costfuncARIMA(SEXP ARorders, SEXP MAorders, SEXP Iorders, SEXP A
 */
 
 // ##### Script for simulate functions
-List simulator(arma::cube arrayVt, arma::mat matrixerrors, arma::mat matrixot,
-                 arma::cube arrayF, arma::rowvec rowvecW, arma::mat matrixG,
-                 unsigned int obs, unsigned int nseries,
-                 char E, char T, char S, arma::uvec lags) {
+List simulator(arma::cube &arrayVt, arma::mat const &matrixerrors, arma::mat const &matrixot,
+                 arma::cube const &arrayF, arma::rowvec const &rowvecW, arma::mat const &matrixG,
+                 unsigned int const &obs, unsigned int const &nseries,
+                 char const &E, char const &T, char const &S, arma::uvec &lags) {
 
     arma::mat matY(obs, nseries);
     arma::rowvec rowvecXt(1, arma::fill::zeros);
@@ -1906,7 +1959,8 @@ List simulator(arma::cube arrayVt, arma::mat matrixerrors, arma::mat matrixot,
                 matrixVt(j,matrixVt.n_cols-1) = arma::as_scalar(trans(matrixVt(lagrows.row(matrixVt.n_cols-1))));
             }
             if(T=='M'){
-                if(matrixVt(j,1) <= 0){
+                if((matrixVt(j,0) <= 0) | (matrixVt(j,1) <= 0)){
+                    matrixVt(j,0) = arma::as_scalar(trans(matrixVt(lagrows.row(0))));
                     matrixVt(j,1) = arma::as_scalar(trans(matrixVt(lagrows.row(1))));
                 }
             }
@@ -1919,7 +1973,7 @@ List simulator(arma::cube arrayVt, arma::mat matrixerrors, arma::mat matrixot,
 
 /* # Wrapper for simulator */
 // [[Rcpp::export]]
-RcppExport SEXP simulatorwrap(SEXP arrvt, SEXP materrors, SEXP matot, SEXP matF, SEXP matw, SEXP matg,
+RcppExport SEXP simulatorwrap(SEXP arrvt, SEXP matErrors, SEXP matot, SEXP matF, SEXP matw, SEXP matg,
                                 SEXP Etype, SEXP Ttype, SEXP Stype, SEXP modellags) {
 
 // ### arrvt should contain array of obs x ncomponents x nseries elements.
@@ -1927,8 +1981,8 @@ RcppExport SEXP simulatorwrap(SEXP arrvt, SEXP materrors, SEXP matot, SEXP matF,
     IntegerVector arrvt_dim = arrvt_n.attr("dim");
     arma::cube arrayVt(arrvt_n.begin(),arrvt_dim[0], arrvt_dim[1], arrvt_dim[2], false);
 
-    NumericMatrix materrors_n(materrors);
-    arma::mat matrixerrors(materrors_n.begin(), materrors_n.nrow(), materrors_n.ncol(), false);
+    NumericMatrix matErrors_n(matErrors);
+    arma::mat matrixerrors(matErrors_n.begin(), matErrors_n.nrow(), matErrors_n.ncol(), false);
 
     NumericMatrix matot_n(matot);
     arma::mat matrixot(matot_n.begin(), matot_n.nrow(), matot_n.ncol(), false);
@@ -1944,8 +1998,8 @@ RcppExport SEXP simulatorwrap(SEXP arrvt, SEXP materrors, SEXP matot, SEXP matF,
     NumericMatrix matg_n(matg);
     arma::mat matrixG(matg_n.begin(), matg_n.nrow(), matg_n.ncol(), false);
 
-    unsigned int obs = materrors_n.nrow();
-    unsigned int nseries = materrors_n.ncol();
+    unsigned int obs = matErrors_n.nrow();
+    unsigned int nseries = matErrors_n.ncol();
     char E = as<char>(Etype);
     char T = as<char>(Ttype);
     char S = as<char>(Stype);
